/***************************************************************************
*  INTPIX DAQ Program
*  
*  File        : mymainframe.h
*  version     : v 1.0.0 2008/11/5
*  Author      : Minoru Hirose
*
*  Description : class MyMainFrame  Methods
*                class MyCanvas     Methods                     
*                class MyOption     Methods                     
*                class MyParameters Methods                     
*                class MylogEdit    Methods                     
*
*  Original Program is made for COBI-INTPIX system by Shun Ono.
*  I appreciate all his work.
***************************************************************************/

#include <TROOT.h>
#include <TApplication.h>
#include <TSystem.h>
#include <TGClient.h>
#include <TCanvas.h>
#include <TF2.h>
#include <TH2.h>
#include <TF1.h>
#include <TH1.h>
#include <TRandom.h>
#include <TGaxis.h>
#include <TGFSContainer.h>
#include <TGFileDialog.h>
#include <TGButtonGroup.h> 
#include <TGButton.h> 
#include <TGLabel.h>
#include <TGMsgBox.h>
#include <TGComboBox.h>
#include <TGFrame.h>
#include <TG3DLine.h>
#include <TGSlider.h>
#include <TGMenu.h>
#include <TGNumberEntry.h>
#include <TGTextEntry.h>
#include <TGTextEdit.h>
#include <TRootEmbeddedCanvas.h>
#include <TStyle.h>
#include <TTimeStamp.h>
#include <TTimer.h>
#include <TMath.h>

#include <iostream>
#include <fstream>
#include <stdlib.h>

using namespace std;

#ifndef __CINT__
#include <string.h>
#include <dirent.h>
#include "calib.h"
#include "MySiTCP.h"
#endif

class MyMenuBar;
class MyCtrl;
class MyCanvas;
class MyOption;
class MyParameters;
class MylogEdit;

class MyMainFrame : public TGMainFrame
{
 private:
  // MyMenuBar *MyMenu;
  MyCtrl *Cont;
  MyCanvas *Canv;
  MyOption *Opt;
  MyParameters *Para;
  MylogEdit *Log;
  TGHorizontalFrame *MainHorzFrame, *SubHorzFrame;
  TGTextEdit *eLog;  
  TGTextButton *reset,*exit;
  TGMsgBox *clMsg, *wrMsg;
  TStyle *myStyle; 
  TF1 *f1;
  TGaxis *revAxis;
  TF2 *f2;
  TH2F *h2[7];
  TH1F *h1[7];
  TTimer *utime, *dtime;
  TTimeStamp *tstmp;
  int totevent, curevent, totcalevent, curcalevent, pedevent, noievent;
  int pauseflg, calibflag;
  int runmode;
  int beforeRSTSig, beforePixScan;
  bool usbret;
  unsigned short raw[128][128], bch[128][128];
  //double sum[128][128], sum_2[128][128];
  double ped[128][128], ino[128][128];
  double data[128][128];
  char *fildirpath, *dataname;
  char datafilepath[1024], logfilepath[1024];
  ofstream *datafout, *logfout;
#ifndef __CINT__  
  Calib *cal;
  DIR *filedir;
  MySiTCP *sitcp;
#endif

 public:
  MyMainFrame(const TGWindow *p,UInt_t w,UInt_t h);
  virtual ~MyMainFrame();
  void Initialize();
  void Connection();
  void RunDAQ();
  void StopDAQ();
  void PauseDAQ();
  void ClearLogText();
  void SaveLogText();
  void DirFind();
  void DAQTimer();
  void UpdateTimer();
  void SetDAQTime();
  void SetUpdateTime();
  void SetHistRange();
  void SetRunMode();
  void SetIntTime();
  void SetRSTSig();
  void SetPixScan();
  void SetRSTV();
  void SetStatus(char *string, unsigned char flags);
  char *GetRltPath(char *fullpath);
  void CloseWindow();
  ClassDef(MyMainFrame,0)
};


class MyMenuBar : public TGMenuBar
{
 protected:
  TGPopupMenu    *MenuFile, *MenuRun;
  TGGroupFrame *EventFrame;
 public:
  MyMenuBar(const TGWindow *p);
  virtual ~MyMenuBar();
  void MenuLayout();
};

class MyCtrl : public TGVerticalFrame
{
 protected:
  TGLabel *NeveLabel, *CeveLabel, *SavDirLabel, *SavFilLabel;
  TGTextButton *start, *stop, *pause;
  TGNumberEntry *NeveEntry;
  TGNumberEntryField *CeveEntry; 
  TGCheckButton *SavDat;
  TGTextEntry *SavFilEntry, *SavDirEntry;
  TGPictureButton *SavDirSel;
 public:
  MyCtrl(const TGWindow* p, UInt_t w, UInt_t h);
  virtual ~MyCtrl();
  void CtrlLayout();
  TGTextButton *GetStart() const { return start;}
  TGTextButton *GetStop() const { return stop;}
  TGTextButton *GetPause() const { return pause;}
  TGNumberEntry *GetNevent() const { return NeveEntry;}
  TGNumberEntryField *GetCevent() const { return CeveEntry;}
  TGTextEntry *GetDEntry() const { return SavDirEntry;}
  TGTextEntry *GetFEntry() const { return SavFilEntry;}
  TGCheckButton *GetSavDat() const { return SavDat;}
  TGPictureButton *GetFDir() const { return SavDirSel;}
};

class MyCanvas : public TGVerticalFrame
{
 protected:
  TGGroupFrame *TimerFrame, *AnglFrame;
  TGHorizontalFrame  *CanvOptFrame, *TimerDAQFrame, *TimerUpdFrame;
  TGLabel *DAQIntLabel, *UpdIntLabel;
  TRootEmbeddedCanvas *ECanvas;
  TGNumberEntry *UpdIntEntry;
  TGComboBox *DAQIntCBox;
 public:
  MyCanvas(const TGWindow* p, UInt_t w, UInt_t h);
  virtual ~MyCanvas();
  void CanvLayout();
  TRootEmbeddedCanvas *GetECanvas() const { return ECanvas;}
  TGNumberEntry *GetUpdInt() const { return UpdIntEntry;}
  TGComboBox *GetDAQIntCBox() const { return DAQIntCBox;} 
};

class MyOption : public TGVerticalFrame
{
 protected:
  TGHorizontalFrame *HistRangeEFrame, *HistRangeLFrame;
  TGVerticalFrame *RunVertical;
  TGGroupFrame *DOptFrame, *HistRangeGroup;
  TGLabel *DOptLabel, *HContLabel, *CEntLabel, *RangeHiLabel, *RangeLowLabel;
  TGComboBox *DOptCBox, *HContCBox;
  TGCheckButton *LogX, *LogY, *LogZ;
  TGNumberEntry *CalibEntry;
  TGNumberEntry *HistHiEntry, *HistLowEntry;
  TGButtonGroup *TakePedeRadio;
  TGRadioButton *fR[3];
  
 public:
  MyOption(const TGWindow* p, UInt_t w, UInt_t h);
  virtual ~MyOption();
  void OptLayout();
  TGComboBox *GetDispOpt() const { return DOptCBox;} 
  TGComboBox *GetHistCont() const { return HContCBox;}
  TGNumberEntry *GetCalib() const { return CalibEntry;}
  TGCheckButton *GetLogX() const  { return LogX;}
  TGCheckButton *GetLogY() const  { return LogY;}
  TGCheckButton *GetLogZ() const  { return LogZ;}

  TGNumberEntry *GetHistHi() const { return HistHiEntry;}
  TGNumberEntry *GetHistLow() const { return HistLowEntry;}
  TGButtonGroup *GetRunMode() const { return TakePedeRadio;}
};

class MyParameters : public TGVerticalFrame
{
 protected:
  TGGroupFrame *INTPIXParaFrame;
  TGLabel *IntTimeLabel, *RSTSigLabel, *PixScanLabel, *RSTVLabel;
  TGNumberEntry *IntTimeEntry, *RSTSigEntry, *PixScanEntry, *RSTVEntry;
 public:
  MyParameters(const TGWindow* p, UInt_t w, UInt_t h);
  virtual ~MyParameters();
  void ParaLayout();
  TGNumberEntry *GetIntTime() const { return IntTimeEntry;}
  TGNumberEntry *GetRSTSig() const { return RSTSigEntry;}
  TGNumberEntry *GetPixScan() const { return PixScanEntry;}
  TGNumberEntry *GetRSTV() const { return RSTVEntry;}
};  

class MylogEdit : public TGGroupFrame
{
 protected:
  TGTextEdit *LogEdit;
  TGVerticalFrame *LogCtrlFrame;
  TGTextButton *clearlog, *savelog;
 public:
  MylogEdit(const TGWindow* p, TGString* title);
  virtual ~MylogEdit();
  void EdtLayout();
  TGTextEdit *GetlogEdit() const { return LogEdit;}
  TGTextButton *GetClrLog() const { return clearlog;}
  TGTextButton *GetSaveLog() const { return savelog;}
};

enum MenuCommandId {
  M_FILE_ALLRESET,
  M_FILE_SAVE,
  M_FILE_SAVEAS,
  M_FILE_EXIT,

  M_RUN_START,
  M_RUN_STOP,
  M_RUN_PAUSE,
  M_RUN_RESETCANV
};

